#!/bin/sh
#
# Component: taskd
#
# Description:
#    Creates a child task from another task
#		uses TASKD_JOBID and TASKD_SUBJOBID
#
# Usage:
#    createchildtask.sh [ -h ] -p <prioritytype> -c <command> [ -i <childid> ] [ -t <type> ] [ -d <deltatype> ] [ -n <jobinfo> ] [ -i <childid> ] 
#		 -h               	: prints this help
#	     -p <prioritytype>  : the priority type of the task (as defined in priorities.cfg)
#	     -c <command>       : the command (executable with parameters) of the task
#	     -i <childid>       : the ID of this child task (0, 1, ... ), will be applicated into subjobid 
#	     -t <type>          : the type (sh or class) of the task
#	     -d <deltatype>     : the delta priority type of the task (as defined in priorities.cfg)
#	     -n <jobinfo>       : the information about this job (display name, user, ...) format is #COLID=value#COLID=value...\n, if not given, it will be empty
#		 -r <relpriority>   : relative priority may be given (used when job is from another machine)#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

###############################
# Variables holding Arguments #
###############################

PriorityType=""
Command=""
ChildID=""


##################################
# Functions #
##################################


# Prints usage with optional error message and exit
#
# [$1]  - error message
function usage_()
{
	[ -n "$1" ] && echo "$1" >&2
	cat <<-ENDOFUSAGE
		$0 [ -h ] -p <prioritytype> -c <command> [ -i <childid> ] [ -t <type> ] [ -d <deltatype> ] [ -n <jobinfo> ] [ -r <relpriority> ] 

		 -h               	: prints this help
	     -p <prioritytype>  : the priority type of the task (as defined in priorities.cfg)
	     -c <command>       : the command (executable with parameters) of the task
	     -i <childid>       : the ID of this child task (0, 1, ... ), will be applicated into subjobid 
	     -t <type>          : the type (sh or class) of the task
	     -d <deltatype>     : the delta priority type of the task (as defined in priorities.cfg)
	     -n <jobinfo>       : the information about this job, if not given, it will be empty
	     -r <relpriority>   : relative priority may be given (used when job is from another machine)

	ENDOFUSAGE

	[ -n "$1" ] && exit 1
	exit 0
}

# Prints error message on standard error and exits with exit code
#
# $1 exit code
# $2 error message
#
function errorExit_()
{
	echo "$2" >&2
	exit $1
}


######################################
# Process arguments and create task  #
######################################

PASS_PARAMS=""

while getopts 'hp:c:t:d:a:n:i:r:' OPT; do
	case $OPT in
		h) usage_ ;;
		p) PriorityType="$OPTARG" ; PASS_PARAMS="$PASS_PARAMS -p \"$OPTARG\"" ;;
		c) Command="$OPTARG" ; PASS_PARAMS="$PASS_PARAMS -c '$OPTARG'" ;;
		i) ChildID="$OPTARG" ; PASS_PARAMS="$PASS_PARAMS -i \"$OPTARG\"" ;;
		t) PASS_PARAMS="$PASS_PARAMS -t \"$OPTARG\"" ;;
		d) PASS_PARAMS="$PASS_PARAMS -d \"$OPTARG\"" ;;
		a) PASS_PARAMS="$PASS_PARAMS -a \"$OPTARG\"" ;;
		n) PASS_PARAMS="$PASS_PARAMS -n \"$OPTARG\"" ;;
		r) PASS_PARAMS="$PASS_PARAMS -r \"$OPTARG\"" ;;
		*) usage_ "Invalid Parameter: $OPT" ;;
	esac
done

if [ -z "$PriorityType" ] ; then
	usage_ "PriorityType must be specified"
fi	

if [ -z "$Command" ] ; then
	usage_ "Command must be specified"
fi

RUNCOMMAND="${TASKD_HOME}/createtaskfileapp"

if [ -n "$TASKD_JOBID" ] ; then
	RUNCOMMAND="$RUNCOMMAND -j $TASKD_JOBID"
fi

if [ -n "$TASKD_SUBJOBID" ] ; then
	SUBJOBPART=` echo "${TASKD_SUBJOBID}" | LC_NUMERIC=en_US awk -vFS="_" -vCHILDID="$ChildID" '
		{ 
			level = $1;
			level = level + 1;
			toprint = "" level;
			for( i = 2; i <= NF; i++ ) {
				toprint = toprint "_" $i;
			}	
			if( CHILDID != "" ) {
				toprint = toprint "_" CHILDID;
			}			
			print( toprint );
		}' `
	
	RUNCOMMAND="$RUNCOMMAND -s $SUBJOBPART"
fi

	
eval "$RUNCOMMAND $PASS_PARAMS"
